;PVdBScriptConfig.ahk 
;AutoHotKey script for Personal Video Database scripts config:
;   First argument A_Args[1]= Config file
;   If exists 'PVdBScriptConfig.log' write variables in this file
;CHANGE LOG:
;   V.2018.08.01-VVV_Easy: Added not 'log.txt' condition for use with PVD 0.9.9.21
;   V.2018.05.27-VVV_Easy: in nested comments or a URL value take the second last comment
;   V.2018.05.23-VVV_Easy: First version.
;------------------------------------------------------------
;Script behavior :
;   1) Show the window which allows config the programed scripts
;   2) I user Aply Re-Start PVD en Portable o Debug mode
;------------------------------------------------------------
;Compilation orders
    #NoEnv  ; Recommended for performance and compatibility with future AutoHotkey releases.
    #Warn  ; Enable warnings to assist with detecting common errors.
    #SingleInstance On  ;The word OFF allows multiple instances of the script to run concurrently
    ;#NoTrayIcon ;Disables the showing of a tray icon.
;Script behavior and Global variables
    global ScriptName 
    SplitPath, A_ScriptName,,,, ScriptName, ;Get ScriptName for windows title and log file.
    global s_LogFile:= A_ScriptDir . "\" . ScriptName . ".log"
    ;SetWorkingDir %A_ScriptDir%  ; Ensures a consistent starting directory.
    global s_ScriptWinTitle:= "Personal Video Database MOD Script Configuration"
    global VarNameScript
    global VarNameToken
    global MaxNumberTokens:=1 ; for set the windows height
    ;If not needed arguments show help
    If A_Args.Length()<1 { 
        MsgBox, 16, % ScriptName, % ScriptName ":`nAutoHotKey script for Personal Video Database scripts config.`n`nUsage:`nFirst argument = Config file"
        ExitApp
    }
    global s_IniFile:= A_Args[1]
;global Config Variables Load
    PVDScripts:= Array() ; x.1=Variable Name,  x.2=File Name, x.3=Number of parameters of Infos Tokens, x.4=Idem of Retreive Data, x.5=Idem of parameters of Process Data, x.6=Idem of Behaviour
    ;Load config file
    if FileExist(s_IniFile)="A" {
        FileRead, Text_IniFile, % s_IniFile
    } else {
        MsgBox, 16, % ScriptName, % "Configuration file`n`n" . s_IniFile . "`n`nmissing. Not possible script configuration."    
        ExitApp
    }
    Loop {        
        If Not InStr(Text_IniFile, "<SCRIPT/>", False, 1, A_Index)
            Break
        ScriptBlock:=TextBetween(Text_IniFile, "<SCRIPT/>", "</SCRIPT>", , A_Index)
        ScriptNumber:=A_Index
        Loop, 6 {
            PVDScripts[ScriptNumber, A_Index]:=TextBetween(ScriptBlock, "<I/>", "</I>", , A_Index)
        }
        NumberTokens:=PVDScripts[ScriptNumber, 3]+PVDScripts[ScriptNumber, 4]+PVDScripts[ScriptNumber, 5]+PVDScripts[ScriptNumber, 6]
        If (NumberTokens > MaxNumberTokens)
            MaxNumberTokens:=NumberTokens
        VarNameScript:=PVDScripts[ScriptNumber, 1] ; AHK don't support double %%. A intermediat variable needed.
        ;MsgBox, 16, % ScriptName, % "|" . VarNameScript . "|" . PVDScripts[A_Index, 1] . "|"
        Tokens_%VarNameScript%:= Array() ; x.1=Token string, x.2=Token type
        ; TokeTypes: "Q" = Quoted string.  Example:  SCRIPT_VERSION='1.0.0.0'; //Script version
        ;            "V" = Value string.   Example:  PEOPLE_LIMIT=26; //Limit of number of people retrieved.
        ;            "B" = Boolean.        Example:  GET_FULL_CREDIT=True; //Download Cast & Credit provider page for retreive the info. Otherwise only info of the principal movie page.
        Loop {
            If Not InStr(ScriptBlock, "<TN/>", False, 1, A_Index)
                Break
            Tokens_%VarNameScript%[A_Index, 1]:=TextBetween(ScriptBlock, "<TN/>", "</TN>", , A_Index)
            Tokens_%VarNameScript%[A_Index, 2]:=TextBetween(ScriptBlock, "<TT/>", "</TT>", , A_Index)
            ;MsgBox, 16, % ScriptName, % "|" . VarNameScript . "|" . Tokens_%VarNameScript%[A_Index, 1] . "|" . Tokens_%VarNameScript%[A_Index, 2] . "|"
        }
    }
;Bulding the GUI for input parameters and action variables
    ;General GUI part
	Gui 1: -Disabled +SysMenu -Owner  ; -AlwaysOnTop ; +Owner No tray button but provides a normal title bar. +ToolWindow No tray button but provides a narrower title bar.
	Gui 1: Margin, 5, 5
    Gui 1:Font, s10, Calibri    
	;Gui 1:Color, cBlack ;Black,Silver,Gray,White,Maroon,Red,Purple,Fuchsia,Green,Lime,Olive,Yellow,Navy,Blue,Teal,Aqua
    Gui 1:Font, Bold
    CalculatedHeigth:=MaxNumberTokens*21+25
    Gui 1:Add, Button, x10 y%CalculatedHeigth% w780 gScriptProcess, Save Config!         (Personal Video Database automatic restart applied)
    TabsHeaders:=""
    for i, ele in PVDScripts {
        TabsHeaders:= TabsHeaders . PVDScripts[i,2] . "|"
    }
    CalculatedHeigth:=MaxNumberTokens*21+10
    Gui 1:Add, Tab3, x5 y5 w800 h%CalculatedHeigth% -Wrap, % TabsHeaders
    Gui 1:Font, Bold
    ;Script GUI tab part
    tab:=0
    for j_script, ele in PVDScripts {
        tab++
        VarNameScript:=PVDScripts[j_script,1] ; AHK don't support double %%. A intermediat variable needed.
        ;Reading PVD Script text-----------------------------------------------------------------------------------------------------
        File_%VarNameScript%:= A_ScriptDir . "\" . PVDScripts[j_script,2]
        if FileExist(File_%VarNameScript% . ".psf")="A" {
            Active_%VarNameScript%:= True
            FileRead, Text_%VarNameScript%, % File_%VarNameScript% . ".psf"
        } else if FileExist(File_%VarNameScript% . ".psf.hid")="A" {
            Active_%VarNameScript%:= False
            FileRead, Text_%VarNameScript%, % File_%VarNameScript% . ".psf.hid"
        } else {
            MsgBox, 16, % ScriptName, % "Script file:`n`n" . File_%VarNameScript% . "`n`nmissing. Not possible script configuration."
            ExitApp 
        }
        ;MsgBox, 64, % ScriptName, % Text_%VarNameScript% ;Debug
        ;Reading Getting Tokens values in the script text----------------------------------------------------------------------------------
        for k_token, ele in Tokens_%VarNameScript% {
            VarNameToken:=Tokens_%VarNameScript%[k_token,1] ; AHK don't support double %%. A intermediat variable needed.
            GetTokenValue(Text_%VarNameScript%, Tokens_%VarNameScript%[k_token,1], Tokens_%VarNameScript%[k_token,2], Value_%VarNameScript%%VarNameToken%, Descr_%VarNameScript%%VarNameToken%)
            ;MsgBox, 16, % ScriptName, % "|" . VarNameScript . "|" . VarNameToken . "|" . Value_%VarNameScript%%VarNameToken% . "|" . Descr_%VarNameScript%%VarNameToken% . "|"
        }
        ;Setting PVDScript Tab GUI
        Gui 1:Tab, % j_script ;When change the Tab you must set the absolute position and the Font
        ;PVDScript Infos Tokens
        Gui 1:Font, Normal Bold
        Gui 1:Add, Text, x10 y30 cBlue, % Value_%VarNameScript%SCRIPT_DESC . " V." . Value_%VarNameScript%SCRIPT_VERSION  . " (" . Value_%VarNameScript%SCRIPT_TYPE . ")"
        VarNameTmp:=Active_%VarNameScript% ; AHK don't support double %%. A intermediat variable needed.
        Gui 1:Add, CheckBox, x10 yp+20 cBlue vActive_%VarNameScript% Checked%VarNameTmp%, % "Active"
        Gui 1:Font, Italic
        Gui 1:Add, Text, xp+100 yp+0 cPurple, % Descr_%VarNameScript%SCRIPT_NAME . " - " . Descr_%VarNameScript%SCRIPT_VERSION
        ;PVDScript others Tokens
        k_token:= PVDScripts[j_script,3]+1 ;ByPass the Infos Tokens
        last_token:= PVDScripts[j_script,3]
        Loop, 3 { ;Three kinds of Tokens
            If (A_Index = 1)
                TokenKind:= " Script Retreive Data Config "
            If (A_Index = 2)
                TokenKind:= " Script Process Data Config "
            If (A_Index = 3)
                TokenKind:= " Script Behaviour Config "
            Gui 1:Add, Text, x10 yp+25 cBlue w770 h2 Border,
            Gui 1:Font, Bold
            Gui 1:Add, Text, x25 yp-7 , % TokenKind
            Gui 1:Font, Normal Italic            
            last_token:= last_token+PVDScripts[j_script,A_Index+3]
            while(k_token <= last_token) {
                VarNameToken:=Tokens_%VarNameScript%[k_token,1] ; AHK don't support double %%. A intermediat variable needed.
                VarNameTmp:=Value_%VarNameScript%%VarNameToken%  ; AHK don't support double %%. A intermediat variable needed.
                ;MsgBox, 64, % ScriptName, % "|" . VarNameScript . "|" . VarNameToken . "|" . VarNameTmp . "|" . %VarNameTmp% . "|" ;Debug            
                if Tokens_%VarNameScript%[k_token,2]="B" ; Boolean Token
                    Gui 1:Add, CheckBox, x10 yp+20 vValue_%VarNameScript%%VarNameToken% Checked%VarNameTmp%, % Descr_%VarNameScript%%VarNameToken%    
                if Tokens_%VarNameScript%[k_token,2]="V" { ; Value Token
                    Gui 1:Add, Edit, x10 yp+18 w96 h21 Border +Center vValue_%VarNameScript%%VarNameToken%, % Value_%VarNameScript%%VarNameToken%
                    Gui 1:Add, Text, xp+100 yp+2, % Descr_%VarNameScript%%VarNameToken%
                }
                k_token++
            }
        }
    }
	Gui 1:Show , , %s_ScriptWinTitle%  ;Gui, Show, AutoSize Center
	Return  ;The script wait in the Gui window showed. It continue with the Gui events.

GuiClose:
GuiEscape:
	Gui 1:Destroy
	;MsgBox, 64, % ScriptName, % ScriptName "-(" . s_ScriptAction . "|" . b_DOS_EOLAction . "|" . b_NoBOMAction . "|" . s_FinalCode . ") canceled.`nGood bye, " . A_UserName . "!" 
	ExitApp
ScriptReload:
	Reload
ScriptProcess:
	Gui 1:Submit, NoHide  ; Save the input from the user to each control's associated variable.
    ;Process the script text
    for j_script, ele in PVDScripts {
        VarNameScript:=PVDScripts[j_script,1] ; AHK don't support double %%. A intermediat variable needed.
        ;Reading Getting Tokens values in the script text----------------------------------------------------------------------------------
        for k_token, ele in Tokens_%VarNameScript% {
            VarNameToken:=Tokens_%VarNameScript%[k_token,1] ; AHK don't support double %%. A intermediat variable needed.
            SavedToken:= VarNameToken . TextBetween(Text_%VarNameScript%, VarNameToken, ";", False) . ";"
            If Tokens_%VarNameScript%[k_token,2] = "Q"
                ActualToken:= Tokens_%VarNameScript%[k_token,1] . "  = '" . Value_%VarNameScript%%VarNameToken% . "';"
            If Tokens_%VarNameScript%[k_token,2] = "V"
                ActualToken:= Tokens_%VarNameScript%[k_token,1] . "  = " . Value_%VarNameScript%%VarNameToken% . ";"
            If (Tokens_%VarNameScript%[k_token,2] = "B" and Value_%VarNameScript%%VarNameToken%)
                ActualToken:= Tokens_%VarNameScript%[k_token,1] . "  = True " . ";"
            If Tokens_%VarNameScript%[k_token,2] = "B" and Not Value_%VarNameScript%%VarNameToken%
                ActualToken:= Tokens_%VarNameScript%[k_token,1] . "  = False " . ";"
            Text_%VarNameScript% := StrReplace(Text_%VarNameScript%, SavedToken, ActualToken, , 1)
        }
        ;MsgBox, 16, % ScriptName, % "|" . Text_%VarNameScript% . "|"
        ;Saving PVD Script text-----------------------------------------------------------------------------------------------------
        File_%VarNameScript%:= A_ScriptDir . "\" . PVDScripts[j_script,2]
        try {  ; Asegurate file access
            If FileExist(File_%VarNameScript% . ".psf" )="A"
                FileMove, % File_%VarNameScript% . ".psf", % File_%VarNameScript% ".bak" , True
            If FileExist(File_%VarNameScript% . ".psf.hid" )="A"
                FileMove, % File_%VarNameScript% . ".psf.hid", % File_%VarNameScript% . ".bak" , True
            If Active_%VarNameScript%
                FileAppend, % Text_%VarNameScript%, % File_%VarNameScript% . ".psf"
            If Not Active_%VarNameScript%
                FileAppend, % Text_%VarNameScript%, % File_%VarNameScript% . ".psf.hid"
        }
        catch e {
            MsgBox 16, % ScriptName "-" A_ThisFunc "-(" A_LineNumber ") ", % "Fatal error catched at line " e.Line " in |" e.What "|:`n" e.Message "`nNot possible script configuration."
            ExitApp            
        }
    }
    ;Personal Video Database automatic restart
    ;Detects the PVD running mode
    SplitPath, A_ScriptDir,,PVDFolder,,
    If Not FileExist(PVDFolder . "\log.txt")
        RunSentence:= PVDFolder . "\viddb.exe -portable"    
    FileGetSize, LogFilesize, % PVDFolder . "\log.txt"
    If LogFilesize = 0 ;PVD only in portable mode
        RunSentence:= PVDFolder . "\viddb.exe -portable"
    If LogFilesize > 0 ;PVD in debug mode
        RunSentence:= PVDFolder . "\viddb.exe -debug -portable"
    ;Close PVD
    WinClose,ahk_exe viddb.exe  ;or ahk_class TPVDMain
    ;Restart PVD
    Run, % RunSentence , % PVDFolder
    ExitApp
;SUBRUTINES--------------------------------------------------
GetTokenValue(StackText, TokenName, TokenType,  ByRef TokenValue, ByRef TokenDescription){
           ;Saves in the ByRef variables selon his TokenType the Token values and description. Returns 0 if Token not found.           
           ; TokeTypes: "Q" = Quoted string.  Example:  SCRIPT_VERSION='1.0.0.0'; //Script version
           ;            "V" = Value string.   Example:  PEOPLE_LIMIT=26; //Limit of number of people retrieved.
           ;            "B" = Boolean.        Example:  GET_FULL_CREDIT=True; //Download Cast & Credit provider page for retreive the info. Otherwise only info of the principal movie page.
    If Not InStr(StackText, TokenName) {
        MsgBox 16, % ScriptName "-" A_ThisFunc "-(" A_LineNumber ") ", % "Fatal error in script: " . VarNameScript . "`nTokenType not found: " . VarNameToken          
        ExitApp
    } 
    if (TokenType="Q"){
        TokenValue:=TextBetween(TextBetween(StackText,TokenName,";"),"'","'")
    } else if (TokenType="V"){
        TokenValue:=TextBetween(TextBetween(StackText,TokenName,";") . "`n","=","`n")
    } else if (TokenType="B"){
        TokenValue:=TextBetween(TextBetween(StackText,TokenName,";") . "`n","=","`n")
        If Not (TokenValue="True" or TokenValue="False") {
            MsgBox 16, % ScriptName "-" A_ThisFunc "-(" A_LineNumber ") ", % "Fatal error in script: " . VarNameScript . "`nBoolean Token not 'True' or 'False': " . VarNameToken
            ExitApp
        }
        TokenValue:=StrToBool(TokenValue)
    } else {
            MsgBox 16, % ScriptName "-" A_ThisFunc "-(" A_LineNumber ") ", % "Fatal error in script: " . VarNameScript . "`nTokenType definition: " . VarNameToken
            ExitApp
    }
    TokenDescription:= TextBetween(TextBetween(StackText,TokenName,"`n") . "`n","//","`n")
    if ( InStr(TokenDescription,"//")<>0 )     ;Nested comments or a URL value. Take the last comment
        TokenDescription:= TextBetween(TokenDescription . "`n","//","`n")        
    Return, InStr(StackText,TokenName,true)
}
TextBetween(_string, _before, _after, TrimSpaces:=True, Occurrence := 1){  ;Returns the text between two strings (after #Ocurrences of the _before) without spaces leading or trailing spaces (TrimSpaces).
                                        ;https://autohotkey.com/board/topic/18249-any-function-like-to-get-string-between-2-strings/
                                        ;Not used https://autohotkey.com/board/topic/83712-getting-text-between-two-points/
    b := InStr(_string, _before, true, 1, Occurrence)
	If (b = 0)
		Return
    bl := StrLen(_before)
	a := InStr(_string, _after, true, b + bl)
	If (a = 0)
		Return
	If TrimSpaces
        Return RegExReplace(SubStr(_string, b + bl, a - b - bl), "(^\s*|\s*$)")
    If Not TrimSpaces
        Return SubStr(_string, b + bl, a - b - bl)
}
StrToBool(v) {  ;Returns a True/False string var to a boolean var.
                ;https://autohotkey.com/board/topic/12678-function-variable-conversion-string-to-boolean/
	v := %v%
	Return v
}
;AutoHotKey Snipets------------------------------------------------------------------------
;==========================================================================================

